/****************************************************************************
 * Copyright (c) 2010-2012, EBM WebSourcing - All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
package com.ebmwebsourcing.easycommons.stream;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.junit.Test;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertEquals;

import com.ebmwebsourcing.easycommons.io.IOHelper;

/**
 * @author ofabre
 * 
 */
public class StreamHelperTest {

    @Test
    public void testCopyByteArrayStreams() {
        byte[] testByteArray = "foo".getBytes();
        ByteArrayInputStream inputStream = new ByteArrayInputStream(testByteArray);
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            StreamHelper.copy(inputStream, outputStream);
            assertArrayEquals(testByteArray, outputStream.toByteArray());
        } finally {
            IOHelper.close(inputStream);
            IOHelper.close(outputStream);
        }
    }

    @Test
    public void testCopyBuffuredStreams() {
        byte[] testByteArray = "foo".getBytes();
        BufferedInputStream bufInputStream = new BufferedInputStream(new ByteArrayInputStream(
                testByteArray));
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        BufferedOutputStream bufOutputStream = new BufferedOutputStream(outputStream);
        try {
            StreamHelper.copy(bufInputStream, bufOutputStream);
            assertArrayEquals(testByteArray, outputStream.toByteArray());
        } finally {
            IOHelper.close(bufInputStream);
            IOHelper.close(bufOutputStream);
        }
    }

    @Test
    public void testCopyBuffuredInputStreamsToByteArrayOutputStream() {
        byte[] testByteArray = "foo".getBytes();
        BufferedInputStream bufInputStream = new BufferedInputStream(new ByteArrayInputStream(
                testByteArray));
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try {
            StreamHelper.copy(bufInputStream, outputStream);
            assertArrayEquals(testByteArray, outputStream.toByteArray());
        } finally {
            IOHelper.close(bufInputStream);
            IOHelper.close(outputStream);
        }
    }

    @Test
    public void testCopyByteArrayInputStreamsToBufferedOutputStream() {
        byte[] testByteArray = "foo".getBytes();
        ByteArrayInputStream inputStream = new ByteArrayInputStream(testByteArray);
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        BufferedOutputStream bufOutputStream = new BufferedOutputStream(outputStream);
        try {
            StreamHelper.copy(inputStream, bufOutputStream);
            assertArrayEquals(testByteArray, outputStream.toByteArray());
        } finally {
            IOHelper.close(inputStream);
            IOHelper.close(bufOutputStream);
        }
    }

    @Test
    public void testGetBytesFromByteArrayInputStreams() {
        byte[] testByteArray = "foo".getBytes();
        ByteArrayInputStream inputStream = new ByteArrayInputStream(testByteArray);
        try {
            assertArrayEquals(testByteArray, StreamHelper.getBytes(inputStream));
        } finally {
            IOHelper.close(inputStream);
        }
    }

    @Test
    public void testGetBytesFromBufferedInputStreams() {
        byte[] testByteArray = "foo".getBytes();
        BufferedInputStream bufInputStream = new BufferedInputStream(new ByteArrayInputStream(
                testByteArray));
        try {
            assertArrayEquals(testByteArray, StreamHelper.getBytes(bufInputStream));
        } finally {
            IOHelper.close(bufInputStream);
        }
    }

    @Test
    public void testToString() throws IOException {
        StringBuilder bigStringBuilder = new StringBuilder();
        for (int i = 0; i < 100; i++) {
            bigStringBuilder.append("ALJAFMKLJAMLFJAMLJFALMJFA \t MLJFMAJFMAJFMJ \n");
        }
        String originalString = bigStringBuilder.toString();
        InputStream is = new ByteArrayInputStream(originalString.getBytes());
        assertEquals(originalString,StreamHelper.getString(is));
    }
}
